from abc import ABC, abstractmethod
from typing import List, Dict, Any
from gru.agents.tools.core.vector_db.models import VectorDBConfig


class VectorDBClient(ABC):
    @abstractmethod
    def connect(self, config: VectorDBConfig) -> None:
        """Establishes a connection to the vector database."""
        pass

    @abstractmethod
    async def similarity_search(self, query: str, top_k: int) -> List[Dict[str, Any]]:
        """Performs a similarity search using the query and returns top-k results."""
        pass

    @abstractmethod
    async def filtered_search(
        self, query: str, filters: Dict[str, Any]
    ) -> List[Dict[str, Any]]:
        """Performs a filtered search based on additional query parameters."""
        pass

    @abstractmethod
    async def add_to_collection(self, collection_name: str, documents: List[Dict[str, Any]]):
        """Adds new documents (conversations or other data) to the vector database collection."""
        pass

    @abstractmethod
    async def update_collection(self, collection_name: str, document_id: str, update_data: Dict[str, Any]):
        """Updates an existing document in the specified collection."""
        pass

    @abstractmethod
    async def create_embedding(self, text: str) -> List[float]:
        """Generates an embedding for the given text."""
        pass

    @abstractmethod
    async def delete_from_collection(self, collection_name: str, document_id: str):
        """Deletes a document from the specified collection."""
        pass

    @abstractmethod
    async def list_collections(self) -> List[str]:
        """Lists all available collections in the vector database."""
        pass

    @abstractmethod
    async def get_collection_stats(self, collection_name: str) -> Dict[str, Any]:
        """Returns statistics for a given collection (e.g., document count, embedding dimensions)."""
        pass

    @abstractmethod
    async def reset_collection(self, collection_name: str):
        """Clears all data from the specified collection."""
        pass



import ast
from typing import Optional, Type
from github import Github, Auth
from langchain_core.tools import BaseTool
from pydantic import BaseModel, Field
from langchain_core.callbacks import CallbackManagerForToolRun


class PythonArgumentsToolInput(BaseModel):
    repository: str = Field(description="repository containing the python file")
    file_path: str = Field(description="path of the python file in the repository")


class PythonArgumentsTool(BaseTool):
    name: str = "get_python_file_arguments"
    description: str = (
        "Use this to determine the command line arguments that are required to execute a python file present in a repository"
    )
    args_schema: Type[BaseModel] = PythonArgumentsToolInput
    return_direct: bool = True

    class Config:
        extra = "allow"

    def __init__(self, token: str):
        super().__init__(token=token)
        self.token = token

    def _run(self, *args, **kwargs):
        return super()._run(*args, **kwargs)
    
    async def _arun(
        self,
        repository: str,
        file_path: str,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ):
        auth = Auth.Token(self.token)
        g = Github(auth=auth)
        repo = g.get_repo(repository)

        contents = repo.get_contents(file_path)
        source = contents.decoded_content.decode()

        arg_details = self.analyze_cmdline_args(source)

        return arg_details

    def analyze_cmdline_args(self, source: str):

        tree = ast.parse(source)

        argparse_details = []

        class ArgumentVisitor(ast.NodeVisitor):

            def is_argparse_constructor(self, node):
                return (
                    isinstance(node.func, ast.Attribute)
                    and isinstance(node.func.value, ast.Name)
                    and node.func.value.id == "argparse"
                    and node.func.attr == "ArgumentParser"
                )

            def extract_argument_values(self, node):
                return [arg.value for arg in node.args if isinstance(arg, ast.Constant)]

            def collect_add_arguments(self, tree):
                for parent in ast.walk(tree):
                    if (
                        isinstance(parent, ast.Call)
                        and isinstance(parent.func, ast.Attribute)
                        and parent.func.attr == "add_argument"
                    ):
                        args = self.extract_argument_values(parent)
                        if args:
                            argparse_details.extend(args)

            def visit_Call(self, node):
                if self.is_argparse_constructor(node):
                    self.collect_add_arguments(tree)

        visitor = ArgumentVisitor()
        visitor.visit(tree)

        return argparse_details



from gru.agents.tools.core.code_generator.models import RetrievalResult
from gru.agents.tools.core.context_retriever.base import ContextRetriever
from langchain_core.vectorstores import VectorStore, InMemoryVectorStore
from langchain_openai.embeddings import OpenAIEmbeddings
from typing import Optional, List, Dict, Any


class DefaultContextRetriever(ContextRetriever):
    def __init__(
        self,
        vector_store: VectorStore,
        system_prompt: Optional[str] = None,
        user_prompt_template: Optional[str] = None,
    ):
        """
        Initializes the Default Context Retriever with an optional prompt.

        :param vector_store: Vector database store for retrieval.
        :param system_prompt: Optional system-level prompt for LLM-based retrieval.
        :param user_prompt_template: Optional user prompt template for querying context.
        """
        self.embeddings = OpenAIEmbeddings()
        vector_db = vector_store or InMemoryVectorStore(embedding=OpenAIEmbeddings())
        super().__init__(vector_store=vector_db, system_prompt=system_prompt, user_prompt_template=user_prompt_template)

    async def initialize_store(self, tables: List[Dict[str, Any]]):
        # Implementation
        raise NotImplementedError("Method not implemented yet")

    async def get_relevant_datasources(self, query: str, top_k: int) -> List[str]:
        # Implementation
        raise NotImplementedError("Method not implemented yet")

    async def get_relevant_schemas(self, tables: List[str]) -> List[str]:
        # Implementation
        raise NotImplementedError("Method not implemented yet")

    async def get_similar_examples(self, query: str) -> List[str]:
        # Implementation
        raise NotImplementedError("Method not implemented yet")

    async def get_documentation(self, query: str) -> List[str]:
        # Implementation
        raise NotImplementedError("Method not implemented yet")

    async def store_conversation(self, query: str, sql: str):
        # Implementation
        raise NotImplementedError("Method not implemented yet")
    
    async def retrieve_context(self, query: str, top_k: int = 5) -> RetrievalResult:
        """
        Retrieves relevant context for the query by fetching data from the vector database.
        """
        tables = await self.get_relevant_datasources(query, top_k)
        schemas = await self.get_relevant_schemas(tables)
        examples = await self.get_similar_examples(query)
        docs = await self.get_documentation(query)

        return RetrievalResult(
            tables=tables,
            schemas=schemas,
            documentation=docs,
            examples=examples,
            low_cardinality_values=[],
            domain_knowledge=[],
            opt_rules=[],
        )




from abc import ABC, abstractmethod
from typing import List, Dict, Any, Optional
from langchain_core.vectorstores import VectorStore
from gru.agents.tools.core.code_generator.models import RetrievalResult


class ContextRetriever(ABC):

    def __init__(self, vector_store: VectorStore ,system_prompt: Optional[str] = None, user_prompt_template: Optional[str] = None):
        """
        Base class for Context Retrievers.

        :param system_prompt: Optional system prompt (default: None).
        :param user_prompt_template: Optional user prompt template (default: None).
        """
        self.vector_store = vector_store
        self.system_prompt = system_prompt
        self.user_prompt_template = user_prompt_template
        
    @abstractmethod
    async def initialize_store(self, tables: List[Dict[str, Any]]):
        pass

    @abstractmethod
    async def get_relevant_datasources(self, query: str, top_k: int) -> List[str]:
        pass

    @abstractmethod
    async def get_relevant_schemas(self, tables: List[str]) -> List[str]:
        pass

    @abstractmethod
    async def get_similar_examples(self, query: str) -> List[str]:
        pass

    @abstractmethod
    async def get_documentation(self, query: str) -> List[str]:
        pass

    @abstractmethod
    async def store_conversation(self, query: str, sql: str):
        pass

    @abstractmethod
    async def retrieve_context(self, query: str, top_k: int) -> RetrievalResult:
        pass



from gru.agents.tools.core.vector_db.base import VectorDBClient
from gru.agents.tools.core.vector_db.models import VectorDBConfig
from typing import List, Dict, Any


class ChromaDBClient(VectorDBClient):
    def connect(self, config: VectorDBConfig) -> None:
        raise NotImplementedError("Method not implemented yet")

    async def similarity_search(self, query: str, top_k: int) -> List[Dict[str, Any]]:
        raise NotImplementedError("Method not implemented yet")

    async def filtered_search(
        self, query: str, filters: Dict[str, Any]
    ) -> List[Dict[str, Any]]:
        raise NotImplementedError("Method not implemented yet")

    async def add_to_collection(self, collection_name: str, documents: List[Dict[str, Any]]):
        raise NotImplementedError("Method not implemented yet")

    async def update_collection(self, collection_name: str, document_id: str, update_data: Dict[str, Any]):
        raise NotImplementedError("Method not implemented yet")

    async def create_embedding(self, text: str) -> List[float]:
        raise NotImplementedError("Method not implemented yet")

    async def delete_from_collection(self, collection_name: str, document_id: str):
        raise NotImplementedError("Method not implemented yet")

    async def list_collections(self) -> List[str]:
        raise NotImplementedError("Method not implemented yet")

    async def get_collection_stats(self, collection_name: str) -> Dict[str, Any]:
        raise NotImplementedError("Method not implemented yet")

    async def reset_collection(self, collection_name: str):
        raise NotImplementedError("Method not implemented yet")



from typing import Optional, Type
from github import Github, Auth
from langchain_core.tools import BaseTool
from pydantic import BaseModel, Field
from langchain_core.callbacks import CallbackManagerForToolRun


class GitRepoContentToolInput(BaseModel):
    repository: str = Field(
        description="repository of which the contents are to be retrieved"
    )


class GitRepoContentRetriever(BaseTool):
    name: str = "get_git_repo_contents"
    description: str = (
        "Use this to get file paths of all the files in a github repository"
    )
    args_schema: Type[BaseModel] = GitRepoContentToolInput
    return_direct: bool = True

    class Config:
        extra = "allow"

    def __init__(self, token: str):
        super().__init__(token=token)
        self.token = token

    def _run(self, *args, **kwargs):
        return super()._run(*args, **kwargs)

    async def _arun(
        self,
        repository: str,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ):
        auth = Auth.Token(self.token)
        g = Github(auth=auth)
        repo = g.get_repo(repository)

        contents = repo.get_contents("")

        result = []
        while contents:
            file_content = contents.pop(0)
            if file_content.type == "dir":
                contents.extend(repo.get_contents(file_content.path))
            else:
                result.append(file_content.path)

        return result



import setuptools

setuptools.setup(
    name="gru",
    version="0.0.1rc25",
    install_requires=[
        "requests>=2.31.0",
        "typing==3.7.4.3",
        "chardet==5.1.0",
        "click>=8.1.7",
        "fire>=0.7.0",
        "pydantic>=2.6.3",
        "cookiecutter>=2.6.0",
        "langgraph>=0.2.50",
        "fastapi>=0.115.5",
        "uvicorn>=0.32.0",
        "langgraph-checkpoint-postgres>=2.0.3",
        "psycopg[binary,pool]",
        "celery>=5.5.0rc4", # TODO: change to 5.5.0 stable version as soon as it is released
        "redis>=5.2.0",
        "websockets>=14.1",
        "termcolor>=2.5.0",
        "PyGithub>=2.5.0",
        "pika==1.3.2"
    ],
    entry_points={
        "console_scripts": [
            "yugenml = gru.cookiecutter.mlops_templates_cli:mlops_template_cli",
            "yserve = gru.ml_serving.server:serve",
            "gru = gru.cli:main",
            "canso = gru.canso_cli:main",
        ],
    },
    include_package_data=True,
    packages=setuptools.find_packages(),
)
